/***************************************************************************
    Description          : KPuzzle - A KDE Jigsaw Puzzle Game
    Version              : 0.2
    Copyright            : (C) 2000-2001 by Michael Wand
    EMail                : mwand@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPUZZLE_H
#define KPUZZLE_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapp.h>
#include <kmainwindow.h>

#include "kpuzzleview.h"

/* The main widget class, derived from KMainWindow */
class KPuzzle : public KMainWindow
{
    Q_OBJECT

public:
	/* Constructor */    
	KPuzzle();

	/* Destructor */
	virtual ~KPuzzle();

	/* Data */
public:
	/* This property defines if the pieces are shown rectangular,
	 * or if they are shaped like real jigsaw pieces.
	 */
	bool usePieceMask() { return _usePieceMask; }

	/* Defines the mode how the piece turning buttons work */
	bool turnPiecesAround() { return _turnPiecesAround; }

	/* The image loading mode - see main.h for possible values */
	int imageLoading() { return _imageLoading; }
	
	/* Name of the last player */
	QString& playerName() { return _playerName; }

protected:
	/* This property defines if the pieces are shown rectangular,
	 * or if they are shaped like real jigsaw pieces.
	 */
	bool _usePieceMask;

	/* Defines the mode how the piece turning buttons work */
	bool _turnPiecesAround; 

	/* The image loading mode - see main.h for possible values */
	int _imageLoading;
	
	/* Name of the last player */
	QString _playerName;

private:
	/* The central widget of this main window */
	KPuzzleView* _centralView;


/* Implementation */
protected:
	/* This function creates all menu and toolbar entries */
	void createGUI();

private slots:
	/* Display text on the status bar */
	void changeStatusbar(const QString& text);

protected slots:
	/* Show options dialog */
	void slotSetOptions();

	/* Show highscores */
	void slotHighscores();
};

#endif /* KPUZZLE_H */
